/*! 
 * \file LU_matrix.h
 * \brief LU method header file
 * 
 * \author Joan Gimeno
 * \date 19/01/2014 (start)
 * \version 1.0
 *
 * This file contains the header for ::LU
 */

#ifndef LU_MATRIX_H
#define LU_MATRIX_H

#include "permutation.h"

/*!
 * \ingroup LU
 * \def WHITOUT_PIVOTING
 * \brief LU decomposition without any pivoting
 * \hideinitializer
 */
#define WHITOUT_PIVOTING 0

/*!
 * \ingroup LU
 * \def PARTIAL_PIVOTING_ROW
 * \brief Partial pivoting by rows in LU decomposition
 * \hideinitializer
 */
#define PARTIAL_PIVOTING_ROW 1

/*!
 * \ingroup LU
 * \def PARTIAL_PIVOTING_COL
 * \brief Partial pivoting by columns in LU decomposition
 * \hideinitializer
 */
#define PARTIAL_PIVOTING_COL 2

/*!
 * \ingroup LU
 * \def FULL_PIVOTING
 * \brief Complete pivoting by rows and columns in LU decomposition
 * \hideinitializer
 */
#define FULL_PIVOTING 3

/* Function prototypes */
void * lu(size_t m, size_t n, double ** const A, double ** const LU, size_t * const pr, size_t * const pc, double tol);
#endif