/*! 
 * \file linear_solve.h
 * \brief Linear solve header file
 * 
 * \author Joan Gimeno
 * \date 22/11/2013 (start)
 * \version 1.0
 *
 * This file contains the header for ::linear_solve
 */

#ifndef LINEAR_SOLVE_H
#define LINEAR_SOLVE_H

#include "QR_matrix.h"
#include "LU_matrix.h"

/*!
 * \def SOLVE_WITH_LU
 * \brief Pointer to a solve a linear system with LU decomposition 
 * \hideinitializer
 */
#define SOLVE_WITH_LU &solve_lu
/*!
 * \def SOLVE_WITH_QR
 * \brief Pointer to a solve a linear system with QR decomposition
 * \hideinitializer
 */
#define SOLVE_WITH_QR &solve_qr
/*!
 * \def _SOLVE_PROTOTYPE
 * \brief Prototype of a function pointer that solves a linear system
 * \hideinitializer
 */
#define _SOLVE_PROTOTYPE \
void * (*solve)(size_t, size_t, double ** const, double * const, double * const, double)

typedef void * (*solve_ptr)(size_t, size_t, double ** const, double * const, double * const, double);

/* Function prototypes */
void forward_substitution_ones(size_t n, double ** const L, double * const x, double * const b);
void * forward_substitution(size_t n, double ** const L, double * const x, double * const b, double tol);
void * back_substitution(size_t n, double ** const U, double * const x, double * const b, double tol);

void * solve_qr(size_t m, size_t n, double ** const A, double * const x, double * const b, double tol);
void * solve_lu(size_t m, size_t n, double ** const A, double * const x, double * const b, double tol);

solve_ptr get_solve_with_lu();
solve_ptr get_solve_with_qr();
#endif