/*! 
 * \file newton.h
 * \brief Header of compute a solutions of an equation with Newton method
 *
 * \author Joan Gimeno
 * \date 22/02/2014 (start)
 * \version 1.0
 */

#ifndef NEWTON_H
#define NEWTON_H

#ifdef DEBUG
#define _NEWTON_DEBUG
#endif

#include "linear_solve.h"

/*!
 * \def OPENMP_NEWTON_METHOD
 * \brief Minimum value for using the openmp library in Newton method
 * \showinitializer
 */
#define _OPENMP_NEWTON_METHOD 1e2
/*!
 * \def _FUNCTION_PROTOTYPE
 * \brief Function prototype of a pic struct
 * \hideinitializer
 */
#define _FUNCTION_PROTOTYPE \
void (*f)(size_t, size_t, double * const, double * const, double ** const, double, double)

/* Function prototypes */

unsigned int get_newton_max_iterations();
unsigned int set_newton_max_iterations(unsigned int newton_max_iter);

void * newton_execute(size_t n, size_t N, double * const x, double * const X, double ** const DF, 
                      double w, double tol,
                      void (*f)(size_t, size_t, double * const, double * const, double ** const, double, double),
                      void * (*solve)(size_t, size_t, double ** const, double * const, double * const, double),
                      short unsigned int autonomous
                     );

// void differential_block(size_t m, size_t n, double * const x, 
//                       double * const f, double ** const df, 
//                       double w, double t);

void newton_differential(size_t n, size_t N, double * const x, double * const X, double ** const DF, 
                void (*f)(size_t, size_t, double * const, double * const, double ** const, double, double),
                double w, double * h);

void dft_r2r_1d(size_t n, size_t N, double * const dft, double * const x, double theta);

int newton(size_t m, size_t n, double * const x, double * const f, double ** const df,
           void * (*solve)(size_t, size_t, double ** const, double * const, double * const, double),
           double tol);
#endif