// #include <math.h>
// #include <stdio.h>
// #include <stdlib.h>

#include "utils.h"

int qrsis(int n, int m, double ** a, double * b, double tol)
/*
   resolucio d'un sistema d'equacions lineals usant el metode qr.

   parametres input:
   a:   matriu nxm, amb n>=m.
   b:   terme independent.
   n:   numero d'equacions.
   m:   numero d'incognites.
   tol: tolerancia per decidir si la matriu es singular o no.

   parameteres output:
   a:  conte basura.
   b:  conte la solucio del sistema.
*/
{
  void qrdes(double **a, int n, int m, double *d);
  int resq(double **a, int n, double *d, double *b, double tol);
  int resr(double **a, int m, double *d, double *b, double tol);
  double *d;
  int i,j,k;
  if (n<m) {printf("qrsis: hi han %d equacions i %d incognites\n",n,m); exit(1);} 
  d=(double*)malloc(m*sizeof(double));
  if (d == NULL) {puts("qrsis: no hi ha prou memoria."); exit(1);}
  if (n == m)
    {
        qrdes(a,n,n,d);
        k=resq(a,n,d,b,tol);
    }
  else
  {
      for (j=0; j<m; j++)
      {
        d[j]=0.e0;
        for (i=0; i<n; i++) d[j] += a[i][j]*b[i];
      }
      for (j=0; j<m; j++) b[j]=d[j];
      qrdes(a,n,m,d);
      k=resr(a,m,d,b,tol);
  }
  free(d);
  return k;
}
int resq(double **a, int n, double *d, double *b, double tol)
{
   int i,k;
   double s;
   k=0;
   for (i=0; i<n-1; i++) {if (fabs(d[i]) < tol) ++k;}
   if (fabs(a[n-1][n-1]) < tol) ++k;
   if (k != 0) return 0;
   for (k=0; k<n-1; k++)
   {
      s=0.e0;
      for (i=k; i<n; i++) s += a[i][k]*b[i];
      s *= -2.e0;
      for (i=k; i<n; i++) b[i] += a[i][k]*s;
   }
   b[n-1] /= a[n-1][n-1];
   for (k=n-2; k>=0; k--)
   {
      s=0.e0;
      for (i=k+1; i<n; i++) s += a[k][i]*b[i];
      b[k]=(b[k]-s)/d[k];
   }
   return 1;
}
int resr(double **a, int m, double *d, double *b, double tol)
{
   int i,k;
   double s;
   k=0;
   for (i=0; i<m; i++) {if (fabs(d[i]) < tol) ++k;}
   if (k != 0) return 0;
   b[0] /= d[0];
   for (k=1; k<m; k++)
   {
      s=0.e0;
      for (i=0; i<k; i++) s += a[i][k]*b[i];
      b[k] = (b[k]-s)/d[k];
   }
   b[m-1] /= d[m-1];
   for (k=m-2; k>=0; k--)
   {
      s=0.e0;
      for (i=k+1; i<m; i++) s += a[k][i]*b[i];
      b[k]=(b[k]-s)/d[k];
   }
   return 1;
}

void qrdes(double **a, int n, int m, double *d)
/*
   factoritzacio qr d'una matriu real qualsevol.

   parametres input:
   a: matriu nxm, amb n>=m.
   n: veure a.
   m: veure a.

   parametres output:
   a: triangle inferior+diagonal = els vectors w que apareixen al
                                   fer householder.
      triangle superior = la corresponent part de la r.
   d: diagonal de la r (d ha de tenir al menys m components).

   avis->si la matriu es quadrada, l'ultim element de la diagonal
         de la r esta a a(n,n). a d(n) NO s'hi posa res.
*/
{
   int i,j,k,kmax;
   double s,mu,*w;
   w=(double*)malloc(m*sizeof(double));
   if (w == NULL) {puts("qrdes: no hi ha prou memoria."); exit(1);}
   kmax = (n == m) ? n-1 : m ;
   for (k=0; k<kmax; k++)
   {
      s=0.e0;
      for (i=k; i<n; i++) s += a[i][k]*a[i][k];
      s=sqrt(s);
      if (a[k][k] > 0.e0) s = -s;
      d[k]=s;
      mu=1.e0/sqrt(2.e0*s*(s-a[k][k]));
      a[k][k]=mu*(a[k][k]-s);
      for (j=k+1; j<n; j++) a[j][k] *= mu;
      for (j=k+1; j<m; j++)
      {
         w[j]=0.e0;
         for (i=k; i<n; i++) w[j] += a[i][k]*a[i][j];
      }
      for (j=k+1; j<m; j++)
      {
         s=-2.e0*w[j]; 
         for (i=k; i<n; i++) a[i][j] += s*a[i][k];
      }
   }
   free(w);
   return;
}
