#include "paraview.h"

#define PARAVIEW_VERSION "4.0.1"

int save_flow(size_t dim, size_t N, size_t M, double * v, FILE * const f) {
  size_t i, j, k;
  double hj;
  
  hj = 2 * M_PI / M;
  
  for (j = 0; j < M; j ++) {    
    for (i = 0; i < N; i ++) {
      if (fprintf(f, "\n%24.16e", j * hj) < 0)
        return 0;
      
      for (k = 0; k < dim; k ++) {
        if (fprintf(f, " %24.16e", v[k]) < 0) // v[k + j*m + i*n]
          return 0;
      }
      
      if (fprintf(f, "\n%24.16e", j * hj) < 0)
        return 0;
      
      for (k = 0; k < dim; k ++, v ++) {
        if (fprintf(f, " %24.16e", *v) < 0) // v[k + j*m + i*n]
          return 0;
      }
    }
  }
  return 1;
}

int saveas_flow(size_t dim, size_t N, size_t M, double * v, char * const name) {
  FILE *f;
  
  if ((f = fopen(name, "w")) == NULL)
    return 0;
  
  if (fprintf(f, "# vtk DataFile Version %s\n%s\nASCII\n\nDATASET STRUCTURED_GRID\nDIMENSIONS %zu %zu 1\nPOINTS %zu float", PARAVIEW_VERSION, name, 2 * N, M, 2 * N * M) < 0)
    return 0;
  
  save_flow(dim, N, M, v, f);
  
  #ifdef _PARAVIEW_DEBUG
  printf("Successful save the paraview file %s\n", name);
  #endif
  
  return fclose(f) != EOF;
}

int save_surface(size_t dim, size_t N, size_t M, double * v, FILE * const f) {
  size_t i, j, k;
  double hj;
    
  hj = 2 * M_PI / M;
  
  for (j = 0; j < M; j ++) {    
    for (i = 0; i < N; i ++) {
      if (fprintf(f, "\n%24.16e", j * hj) < 0)
        return 0;
      
      for (k = 0; k < dim; k ++, v ++)
        if (fprintf(f, " %24.16e", *v) < 0) // v[k + j*m + i*n]
          return 0;
    }
  }
  
  return 1;
}

int saveas_surface(size_t dim, size_t N, size_t M, double * v, char * const name) {
  FILE *f;
  
  if ((f = fopen(name, "w")) == NULL)
    return 0;
  
  if (fprintf(f, "# vtk DataFile Version %s\n%s\nASCII\n\nDATASET STRUCTURED_GRID\nDIMENSIONS %zu %zu 1\nPOINTS %zu float", PARAVIEW_VERSION, name, N, M, N * M) < 0)
    return 0;
  
  save_surface(dim, N, M, v, f);
  
  #ifdef _PARAVIEW_DEBUG
  printf("Successful save the paraview file %s\n", name);
  #endif
  
  return fclose(f) != EOF;
}