#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <time.h>

#define MAX_LENGTH_STRING 64
#define EPS 1e-10

int pseudo_random_examples(size_t m, size_t n, FILE * const fm, FILE * const fv) {
  typeof(m) i, j;
  double tmp;
  
  time_t t;
  srand((unsigned) time(&t));
  
  for (i = 0; i < m; i ++) {
    tmp = ((double) rand()) / RAND_MAX;
    
    if (tmp >= EPS)
      if (fprintf(fv, "%zu %24.16e\n", i, tmp) < 0)
        return 0;
      
    for (j = 0; j < n; j ++) {
      tmp = ((double) rand()) / RAND_MAX;
      if (tmp >= EPS)
        if (fprintf(fm, "%zu %zu %24.16e\n", i, j, tmp) < 0)
          return 0;
    }
  }
  
  return 1;
}

void print_usage(char * const name) {
  printf("Usage: %s \n\t-h\t\tHelp instruction\n\t-m val\t\tNumber of rows\n\t-n val\t\tNumber of columns\n\t-o name\t\tOutput files names\n", name);
}

int main(int argc, char **argv) {
  FILE *fm, *fv;
  size_t m, n;
  
  /* Default values */
  char output_matrix[MAX_LENGTH_STRING] = "test_matrix.txt";
  char output_vector[MAX_LENGTH_STRING] = "test_vector.txt";
  
  m = n = 10;
  
  /* Evaluate arguments */
  int c;
  while ((c = getopt(argc, argv, "m:n:ho:")) != EOF) {
    switch (c) {
      case 'm':                 /* Rows */
        m = strtoul(optarg, NULL, 10);
        break;
      case 'n':                 /* Columns */
        n = strtoul(optarg, NULL, 10);
        break;
      case 'o':                 /* Columns */
        snprintf(output_matrix, MAX_LENGTH_STRING, "%s_matrix.txt", optarg);
        snprintf(output_vector, MAX_LENGTH_STRING, "%s_vector.txt", optarg);
        break;
      default:
        print_usage(argv[0]);
        return 1;
    }
  }
  if (m == 0) m = n;
  
  /* Open the output files */
  if ((fm = fopen(output_matrix, "w")) == NULL)
    fprintf(stderr, "Unsuccessful open file %s\n", output_matrix);
    return 1;
  if ((fv = fopen(output_vector, "w")) == NULL) {
    fprintf(stderr, "Unsuccessful open file %s\n", output_vector);
    fclose(fm);
    return 1;
  }
  
  /* Generate the examples */
  int r = pseudo_random_examples(m, n, fm, fv);
  
  /* Close the streams */
  r = fclose(fm) != EOF && !r;
  
  return fclose(fv) != EOF && r;
}
